@@ -1,7 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Plugins.Monitors.Batt
--- Copyright   :  (c) 2010, 2011, 2012, 2013 Jose A Ortega
+-- Copyright   :  (c) 2010, 2011, 2012, 2013, 2015, 2016 Jose A Ortega
 --                (c) 2010 Andrea Rossato, Petr Rockai
 -- License     :  BSD-style (see LICENSE)
 --
@@ -21,6 +21,9 @@ import System.FilePath ((</>))
 import System.IO (IOMode(ReadMode), hGetLine, withFile)
 import System.Posix.Files (fileExist)
 import System.Console.GetOpt
+import Data.List (sort, sortOn, group)
+import Data.Maybe (fromMaybe)
+import Text.Read (readMaybe)
 
 data BattOpts = BattOpts
   { onString :: String
@@ -48,8 +51,8 @@ defaultOpts = BattOpts
   , lowWColor = Nothing
   , mediumWColor = Nothing
   , highWColor = Nothing
-  , lowThreshold = -12
-  , highThreshold = -10
+  , lowThreshold = 10
+  , highThreshold = 12
   , onlineFile = "AC/online"
   , scale = 1e6
   , onIconPattern = Nothing
@@ -84,7 +87,7 @@ parseOpts argv =
     (o, _, []) -> return $ foldr id defaultOpts o
     (_, _, errs) -> ioError . userError $ concat errs
 
-data Status = Charging | Discharging | Idle
+data Status = Charging | Discharging | Full | Idle | Unknown deriving (Read, Eq)
 
 data Result = Result Float Float Float Status | NA
 
@@ -101,6 +104,7 @@ data Files = Files
   , fNow :: String
   , fVoltage :: String
   , fCurrent :: String
+  , fStatus :: String
   , isCurrent :: Bool
   } | NoFiles
 
@@ -108,6 +112,7 @@ data Battery = Battery
   { full :: !Float
   , now :: !Float
   , power :: !Float
+  , status :: !String
   }
 
 safeFileExist :: String -> String -> IO Bool
@@ -132,6 +137,7 @@ batteryFiles bat =
                                   , fNow = prefix </> ch ++ "_now"
                                   , fCurrent = prefix </> cf
                                   , fVoltage = prefix </> "voltage_now"
+                                  , fStatus = prefix </> "status"
                                   , isCurrent = not ip}
 
 haveAc :: FilePath -> IO Bool
@@ -140,17 +146,22 @@ haveAc f =
   where onError = const (return False) :: SomeException -> IO Bool
 
 readBattery :: Float -> Files -> IO Battery
-readBattery _ NoFiles = return $ Battery 0 0 0
+readBattery _ NoFiles = return $ Battery 0 0 0 "Idle"
 readBattery sc files =
     do a <- grab $ fFull files
        b <- grab $ fNow files
        d <- grab $ fCurrent files
+       s <- grabs $ fStatus files
        let sc' = if isCurrent files then sc / 10 else sc
-       return $ Battery (3600 * a / sc') -- wattseconds
+           a' = max a b -- sometimes the reported max charge is lower than
+       return $ Battery (3600 * a' / sc') -- wattseconds
                         (3600 * b / sc') -- wattseconds
                         (d / sc') -- watts
+                        s -- string: Discharging/Charging/Full
     where grab f = handle onError $ withFile f ReadMode (fmap read . hGetLine)
           onError = const (return (-1)) :: SomeException -> IO Float
+          grabs f = handle onError' $ withFile f ReadMode hGetLine
+          onError' = const (return "Idle") :: SomeException -> IO String
 
 readBatteries :: BattOpts -> [Files] -> IO Result
 readBatteries opts bfs =
@@ -160,13 +171,13 @@ readBatteries opts bfs =
            ft = sum (map full bats)
            left = if ft > 0 then sum (map now bats) / ft else 0
            watts = sign * sum (map power bats)
-           idle = watts == 0
-           time = if idle then 0 else sum $ map time' bats
-           mwatts = if idle then 1 else sign * watts
+           time = if watts == 0 then 0 else max 0 (sum $ map time' bats)
+           mwatts = if watts == 0 then 1 else sign * watts
            time' b = (if ac then full b - now b else now b) / mwatts
-           acst | idle      = Idle
-                | ac        = Charging
-                | otherwise = Discharging
+           statuses :: [Status]
+           statuses = map (fromMaybe Unknown . readMaybe)
+                          (sort (map status bats))
+           acst = head $ last $ sortOn length (group statuses)
        return $ if isNaN left then NA else Result left watts time acst
 
 runBatt :: [String] -> Monitor String
@@ -178,12 +189,13 @@ runBatt' bfs args = do
   c <- io $ readBatteries opts =<< mapM batteryFiles bfs
   suffix <- getConfigValue useSuffix
   d <- getConfigValue decDigits
+  nas <- getConfigValue naString
   case c of
     Result x w t s ->
       do l <- fmtPercent x
          ws <- fmtWatts w opts suffix d
          si <- getIconPattern opts s x
-         parseTemplate (l ++ [fmtStatus opts s, fmtTime $ floor t, ws, si])
+         parseTemplate (l ++ [fmtStatus opts s nas, fmtTime $ floor t, ws, si])
     NA -> getConfigValue naString
   where fmtPercent :: Float -> Monitor [String]
         fmtPercent x = do
@@ -200,18 +212,22 @@ runBatt' bfs args = do
                                     then minutes else '0' : minutes
           where hours = show (x `div` 3600)
                 minutes = show ((x `mod` 3600) `div` 60)
-        fmtStatus opts Idle = idleString opts
-        fmtStatus opts Charging = onString opts
-        fmtStatus opts Discharging = offString opts
+        fmtStatus opts Idle _ = idleString opts
+        fmtStatus _ Unknown na = na
+        fmtStatus opts Full _ = idleString opts
+        fmtStatus opts Charging _ = onString opts
+        fmtStatus opts Discharging _ = offString opts
         maybeColor Nothing str = str
         maybeColor (Just c) str = "<fc=" ++ c ++ ">" ++ str ++ "</fc>"
         color x o | x >= 0 = maybeColor (posColor o)
                   | -x >= highThreshold o = maybeColor (highWColor o)
                   | -x >= lowThreshold o = maybeColor (mediumWColor o)
                   | otherwise = maybeColor (lowWColor o)
-        getIconPattern opts status x = do
+        getIconPattern opts st x = do
           let x' = minimum [1, x]
-          case status of
+          case st of
+               Unknown -> showIconPattern (offIconPattern opts) x'
                Idle -> showIconPattern (idleIconPattern opts) x'
+               Full -> showIconPattern (idleIconPattern opts) x'
                Charging -> showIconPattern (onIconPattern opts) x'
                Discharging -> showIconPattern (offIconPattern opts) x'
