@@ -16,6 +16,7 @@ module Plugins.PipeReader where
 
 import System.IO
 import Plugins
+import Environment
 import System.Posix.Files
 import Control.Concurrent(threadDelay)
 import Control.Exception
@@ -27,7 +28,7 @@ data PipeReader = PipeReader String String
 instance Exec PipeReader where
     alias (PipeReader _ a)    = a
     start (PipeReader p _) cb = do
-        let (def, pipe) = split ':' p
+        (def, pipe) <- split ':' <$> expandEnv p
         unless (null def) (cb def)
         checkPipe pipe
         h <- openFile pipe ReadWriteMode
@@ -42,4 +43,4 @@ checkPipe file =
     handle (\(SomeException _) -> waitForPipe) $ do
         status <- getFileStatus file
         unless (isNamedPipe status) waitForPipe
-    where waitForPipe = threadDelay 1000 >> checkPipe file
+    where waitForPipe = threadDelay 1000000 >> checkPipe file
